#include <string.h>
#include <strings.h>
#include "MD5.hh"
#include "AuthDigest.hh"

namespace MD5Digest {

void CvtHex(IN HASH Bin, OUT HASHHEX Hex)
{
	unsigned short i;
	unsigned char j;

	for (i = 0; i < HASHLEN; i++) {
		j = (Bin[i] >> 4) & 0xf;
		if (j <= 9)
			Hex[i*2] = (j + '0');
		 else
			Hex[i*2] = (j + 'a' - 10);
		j = Bin[i] & 0xf;
		if (j <= 9)
			Hex[i*2+1] = (j + '0');
		 else
			Hex[i*2+1] = (j + 'a' - 10);
	};
	Hex[HASHHEXLEN] = '\0';
};

/* calculate H(A1) as per spec */
void DigestCalcHA1(
	IN const char * pszAlg,
	IN const char * pszUserName,
	IN const char * pszRealm,
	IN const char * pszPassword,
	IN const char * pszNonce,
	IN const char * pszCNonce,
	OUT HASHHEX SessionKey
	)
{
	  MD5_CTX Md5Ctx;
	  HASH HA1;

	  MD5Init(&Md5Ctx);
	  MD5Update(&Md5Ctx, (const unsigned char*)pszUserName, strlen(pszUserName));
	  MD5Update(&Md5Ctx, (const unsigned char*)":", 1);
	  MD5Update(&Md5Ctx, (const unsigned char*)pszRealm, strlen(pszRealm));
	  MD5Update(&Md5Ctx, (const unsigned char*)":", 1);
	  MD5Update(&Md5Ctx, (const unsigned char*)pszPassword, strlen(pszPassword));
	  MD5Final((unsigned char*)HA1, &Md5Ctx);
	  if (strcasecmp(pszAlg, "md5-sess") == 0) {
			MD5Init(&Md5Ctx);
			MD5Update(&Md5Ctx, (const unsigned char*)HA1, HASHLEN);
			MD5Update(&Md5Ctx, (const unsigned char*)":", 1);
			MD5Update(&Md5Ctx, (const unsigned char*)pszNonce, strlen(pszNonce));
			MD5Update(&Md5Ctx, (const unsigned char*)":", 1);
			MD5Update(&Md5Ctx, (const unsigned char*)pszCNonce, strlen(pszCNonce));
			MD5Final((unsigned char*)HA1, &Md5Ctx);
	  };
	  CvtHex(HA1, SessionKey);
};

/* calculate request-digest/response-digest as per HTTP Digest spec */
void DigestCalcResponse(
	IN HASHHEX HA1,		   /* H(A1) */
	IN const char * pszNonce,	   /* nonce from server */
	IN const char * pszNonceCount,  /* 8 hex digits */
	IN const char * pszCNonce,	  /* client nonce */
	IN const char * pszQop,		 /* qop-value: "", "auth", "auth-int" */
	IN const char * pszMethod,	  /* method from the request */
	IN const char * pszDigestUri,   /* requested URL */
	IN HASHHEX HEntity,	   /* H(entity body) if qop="auth-int" */
	OUT HASHHEX Response	  /* request-digest or response-digest */
	)
{
	  MD5_CTX Md5Ctx;
	  HASH HA2;
	  HASH RespHash;
	   HASHHEX HA2Hex;

	  // calculate H(A2)
	  MD5Init(&Md5Ctx);
	  MD5Update(&Md5Ctx, (const unsigned char*)pszMethod, strlen(pszMethod));
	  MD5Update(&Md5Ctx, (const unsigned char*)":", 1);
	  MD5Update(&Md5Ctx, (const unsigned char*)pszDigestUri, strlen(pszDigestUri));
	  if (strcasecmp(pszQop, "auth-int") == 0) {
			MD5Update(&Md5Ctx, (const unsigned char*)":", 1);
			MD5Update(&Md5Ctx, (const unsigned char*)HEntity, HASHHEXLEN);
	  };
	  MD5Final((unsigned char*)HA2, &Md5Ctx);
	   CvtHex(HA2, HA2Hex);

	  // calculate response
	  MD5Init(&Md5Ctx);
	  MD5Update(&Md5Ctx, (const unsigned char*)HA1, HASHHEXLEN);
	  MD5Update(&Md5Ctx, (const unsigned char*)":", 1);
	  MD5Update(&Md5Ctx, (const unsigned char*)pszNonce, strlen(pszNonce));
	  MD5Update(&Md5Ctx, (const unsigned char*)":", 1);
	  if (*pszQop) {
		  MD5Update(&Md5Ctx, (const unsigned char*)pszNonceCount, strlen(pszNonceCount));
		  MD5Update(&Md5Ctx, (const unsigned char*)":", 1);
		  MD5Update(&Md5Ctx, (const unsigned char*)pszCNonce, strlen(pszCNonce));
		  MD5Update(&Md5Ctx, (const unsigned char*)":", 1);
		  MD5Update(&Md5Ctx, (const unsigned char*)pszQop, strlen(pszQop));
		  MD5Update(&Md5Ctx, (const unsigned char*)":", 1);
	  };
	  MD5Update(&Md5Ctx, (const unsigned char*)HA2Hex, HASHHEXLEN);
	  MD5Final((unsigned char*)RespHash, &Md5Ctx);
	  CvtHex(RespHash, Response);
};


}	// namespace

