/* (C) 2005 Richard Mills / The Linux Academy Ltd.
 *
 * A client side TCP/IP socket server to listen for incoming connections.
 *
 * $Id: ClientServer.cc,v 1.4 2007-06-16 10:43:56 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "ClientServer.hh"
#include "ClientSideConfig.hh"
#include "ClientTunneler.hh"
#include "SOCKSClientTunneler.hh"
#include "Debug.hh"


namespace HTTPTunnel {


// Construction/destruction
ClientServer::ClientServer(ClientSideConfig* clientConfig) throw(TunnelError)
: Server(clientConfig)
{
	DPRINTV("ClientServer::ClientServer()");

	// Create appropriate tunneler
	if(clientConfig->socks()) {
		// Create SOCKS client tunneler
		m_tunneler = new SOCKSClientTunneler(this);
	}
	else {
		// Create plain client tunneler
		m_tunneler = new ClientTunneler(this);
	}

	// Setup server socket
	configureSocket(false);
}

ClientServer::~ClientServer()
{
}


}	// namespace

