/* (C) 2006 Richard Mills / The Linux Academy Ltd.
 *
 * A convenient place to store all global configuration for
 * a server tunneler.
 *
 * $Id: ServerSideConfig.hh,v 1.12 2007-08-18 13:36:26 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _GLOBAL_SERVER_CONFIG_HH_
#define _GLOBAL_SERVER_CONFIG_HH_

#include <string>
#include "SideConfig.hh"
#include "Exceptions.hh"

namespace HTTPTunnel {

class Authenticator;
class ServerAuthenticator;
class Firewall;

class ServerSideConfig : public SideConfig {
public:
	// Construction/destruction
	ServerSideConfig(bool ssl, std::string cert, std::string certKey,
		std::string lhost, unsigned int lport, std::string firewallIPs,
		std::string firewallPorts, bool auth) throw(AuthError);
	virtual ~ServerSideConfig();


	/*************/
	/* Accessors */
	/*************/

	// Server config?
	virtual bool isServer(void) const;

	// Get firewall
	Firewall* getFirewall(void);

	// Get SSL certificate/key
	std::string getCertificate(void) const;
	std::string getCertificateKey(void) const;


	/************/
	/* Mutators */
	/************/

	// Turn authentication on/off
	void setAuthenticate(bool auth) throw(AuthError);


	/*************/
	/* Utilities */
	/*************/

	// Do any required late-construction of data
	virtual void doLateConstruction(void) throw(TunnelError);

	// Serialize to a file
	virtual void serialize(std::ofstream& file);


protected:

	/***********/
	/* Methods */
	/***********/

	// Create authenticator
	void createAuthenticator(void) throw(AuthError);


	/********/
	/* Data */
	/********/

	// Do we use authentication?
	bool							m_useAuth;

	// Firewall
	Firewall*					m_firewall;

	// SSL certificate & private key files
	std::string					m_cert;
	std::string					m_certKey;

};


}	// namespace

#endif
