/* (C) 2006 Richard Mills / The Linux Academy Ltd.
 *
 * Server side Tunnel.
 *
 * $Id: ServerTunnel.hh,v 1.12 2007-11-04 16:17:01 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _SERVER_TUNNEL_HH_
#define _SERVER_TUNNEL_HH_

#include <vector>
#include <string>
#include "Tunnel.hh"

namespace HTTPTunnel {

class ServerTunneler;

class ServerTunnel : public Tunnel {
public:
	// Construction/destruction
	ServerTunnel(TunnelConfig* tcfg, ServerTunneler* tunneler);
	virtual ~ServerTunnel();


	/*************/
	/* Accessors */
	/*************/

	// Get source/destination addresses of connections for this tunnel
	virtual std::string getSrcConnectionAddresses(void);
	virtual std::string getDstConnectionAddresses(void);


	/***********/
	/* Methods */
	/***********/

	// Process some newly extracted tunnel data from one of our connections
	virtual void processTunnelMsg(HTTPMsg* msg, HTTPConnection* httpConn);


protected:

	/***********/
	/* Methods */
	/***********/

	// Create a new payload msg for sending down the tunnel
	virtual HTTPMsgParms* makeTunnelMsg(const HTTPMsgParms* recvd, bool retry)
		throw(MemoryError);

	// Record that we've sent a new tunnel msg
	virtual void sentNewTunnelMsg(HTTPMsgParms* msgParms);

	// Determine whether a multi-ack implies we should resend a msg
	virtual bool checkForMultiAckResendRequest(SentMsgParms* sentMsgParms);

	// Can we send a new tunnel msg?
	virtual bool clearToSendNewMsg(Connection* conn);

	// Check whether any connections need to be blocked/unblocked
	virtual void checkBlockedConnections(void);

	// Advance our internal state machine
	virtual void advanceState(const HTTPMsgParms* parms);

	// Get maximum allowable payload size
	virtual unsigned int getMaxPayloadSize(bool useHeader);


private:

	/*************/
	/* Constants */
	/*************/

	// Minimum client version we allow to connect
	static const unsigned int		MIN_CLIVER_MAJOR		= 1;
	static const unsigned int		MIN_CLIVER_MINOR		= 0;
	static const unsigned int		MIN_CLIVER_POINT		= 1;

	// Maximum no. of authentication failures we allow
	static const unsigned int		MAX_AUTH_FAILURES		= 3;


	/***********/
	/* Methods */
	/***********/

	// Try to authenticate a client
	void authenticateClient(const std::vector<std::string>& authArgs);

	// Is this tunnel user authenticated?
	bool userAuthenticated(void);


	/********/
	/* Data */
	/********/

	// Count of consecutive client authentication failures
	unsigned int						m_authFailureCount;

};


}	// namespace

#endif

