/* (C) 2003 Kevin Manning.
 * (C) 2005 Richard Mills / The Linux Academy Ltd.
 *
 * Exceptions thrown by the thread library
 *
 * $Id: ThreadExceptions.hh,v 1.1 2006-04-01 14:11:05 rich Exp $
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _THREAD_EXCEPTIONS_HH_
#define _THREAD_EXCEPTIONS_HH_

#include <exception>
#include <string>

namespace Threads {

// Base class
class ThreadError : public std::exception {
public:
	ThreadError(std::string e) : err(e) {}
	~ThreadError() throw() {}
	std::string what(void) { return err; }
private:
	std::string err;
};

// Thread not found
class ThreadNotFound : public ThreadError {
public:
	ThreadNotFound(std::string e) : ThreadError(e) {}
};

// Mutex error
class MutexError : public ThreadError {
public:
	MutexError(std::string e) : ThreadError(e) {}
};

// Condition variable error
class ConditionVarError : public ThreadError {
public:
	ConditionVarError(std::string e) : ThreadError(e) {}
};

// Thread specific data error
class ThreadDataError : public ThreadError {
public:
	ThreadDataError(std::string e) : ThreadError(e) {}
};


}		// namespace Threads

#endif

