/* (C) 2007 Richard Mills / The Linux Academy Ltd.
 *
 * Various parameters to be used by a Tunnel when tunneling.
 *
 * $Id: TunnelParms.cc,v 1.3 2007-11-04 16:17:01 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "TunnelParms.hh"


namespace HTTPTunnel {


// Construction/destruction
TunnelParms::TunnelParms(Tunnel::TunnelTestState testState)
: m_createTime(Time::now()), m_testState(testState)
{
}

TunnelParms::TunnelParms(const TunnelParms& that)
: m_createTime(that.m_createTime), m_testState(that.m_testState)
{
}

TunnelParms::~TunnelParms()
{
}


// Assignment
TunnelParms& TunnelParms::operator=(const TunnelParms& that)
{
	// Self assignment
	if(this != &that) {
		m_createTime = that.m_createTime;
		m_testState = that.m_testState;
	}

	return *this;
}


/*************/
/* Accessors */
/*************/


// Get time we were created
const Time& TunnelParms::getCreateTime(void) const
{
	return m_createTime;
}

// Get test state when we were created
Tunnel::TunnelTestState TunnelParms::getTestState(void) const
{
	return m_testState;
}

// Get whether we represent completed tests
bool TunnelParms::testsCompleted(void) const
{
	return (getTestState() == Tunnel::ALL_COMPLETED_TESTING);
}


}	// namespace

